@@ -5,16 +5,43 @@ use crate::error::SparseLinearAlgebraError;
 use crate::operators::{
     binary_operator::BinaryOperator, mask::MatrixMask, options::OperatorOptions,
 };
-use crate::sparse_matrix::SparseMatrix;
 use crate::util::{
     ElementIndex, ElementIndexSelector, ElementIndexSelectorGraphblasType, IndexConversion,
 };
-use crate::value_type::{AsBoolean, ValueType};
+use crate::value_types::sparse_matrix::SparseMatrix;
+use crate::value_types::value_type::{AsBoolean, ValueType};
 
 use crate::bindings_to_graphblas_implementation::{
     GrB_BinaryOp, GrB_Descriptor, GrB_Matrix_extract,
 };
 
+// Implemented methods do not provide mutable access to GraphBLAS operators or options.
+// Code review must consider that no mtable access is provided.
+// https://doc.rust-lang.org/nomicon/send-and-sync.html
+unsafe impl Send for SubMatrixExtractor<bool, bool> {}
+unsafe impl Send for SubMatrixExtractor<u8, u8> {}
+unsafe impl Send for SubMatrixExtractor<u16, u16> {}
+unsafe impl Send for SubMatrixExtractor<u32, u32> {}
+unsafe impl Send for SubMatrixExtractor<u64, u64> {}
+unsafe impl Send for SubMatrixExtractor<i8, i8> {}
+unsafe impl Send for SubMatrixExtractor<i16, i16> {}
+unsafe impl Send for SubMatrixExtractor<i32, i32> {}
+unsafe impl Send for SubMatrixExtractor<i64, i64> {}
+unsafe impl Send for SubMatrixExtractor<f32, f32> {}
+unsafe impl Send for SubMatrixExtractor<f64, f64> {}
+
+unsafe impl Sync for SubMatrixExtractor<bool, bool> {}
+unsafe impl Sync for SubMatrixExtractor<u8, u8> {}
+unsafe impl Sync for SubMatrixExtractor<u16, u16> {}
+unsafe impl Sync for SubMatrixExtractor<u32, u32> {}
+unsafe impl Sync for SubMatrixExtractor<u64, u64> {}
+unsafe impl Sync for SubMatrixExtractor<i8, i8> {}
+unsafe impl Sync for SubMatrixExtractor<i16, i16> {}
+unsafe impl Sync for SubMatrixExtractor<i32, i32> {}
+unsafe impl Sync for SubMatrixExtractor<i64, i64> {}
+unsafe impl Sync for SubMatrixExtractor<f32, f32> {}
+unsafe impl Sync for SubMatrixExtractor<f64, f64> {}
+
 #[derive(Debug, Clone)]
 pub struct SubMatrixExtractor<Matrix, SubMatrix>
 where
@@ -276,7 +303,9 @@ mod tests {
 
     use crate::context::{Context, Mode};
     use crate::operators::binary_operator::First;
-    use crate::sparse_matrix::{FromMatrixElementList, GetMatrixElementValue, MatrixElementList};
+    use crate::value_types::sparse_matrix::{
+        FromMatrixElementList, GetMatrixElementValue, MatrixElementList,
+    };
 
     #[test]
     fn test_matrix_extraction() {
