@@ -5,9 +5,9 @@ use crate::error::SparseLinearAlgebraError;
 use crate::operators::{
     binary_operator::BinaryOperator, mask::MatrixMask, options::OperatorOptions,
 };
-use crate::sparse_matrix::SparseMatrix;
 use crate::util::{ElementIndexSelector, ElementIndexSelectorGraphblasType, IndexConversion};
-use crate::value_type::{AsBoolean, ValueType};
+use crate::value_types::sparse_matrix::SparseMatrix;
+use crate::value_types::value_type::{AsBoolean, ValueType};
 
 use crate::bindings_to_graphblas_implementation::{
     GrB_BinaryOp, GrB_Descriptor, GrB_Matrix_assign,
@@ -15,6 +15,33 @@ use crate::bindings_to_graphblas_implementation::{
 
 // TODO: explicitly define how dupicates are handled
 
+// Implemented methods do not provide mutable access to GraphBLAS operators or options.
+// Code review must consider that no mtable access is provided.
+// https://doc.rust-lang.org/nomicon/send-and-sync.html
+unsafe impl Send for InsertMatrixIntoMatrix<bool, bool> {}
+unsafe impl Send for InsertMatrixIntoMatrix<u8, u8> {}
+unsafe impl Send for InsertMatrixIntoMatrix<u16, u16> {}
+unsafe impl Send for InsertMatrixIntoMatrix<u32, u32> {}
+unsafe impl Send for InsertMatrixIntoMatrix<u64, u64> {}
+unsafe impl Send for InsertMatrixIntoMatrix<i8, i8> {}
+unsafe impl Send for InsertMatrixIntoMatrix<i16, i16> {}
+unsafe impl Send for InsertMatrixIntoMatrix<i32, i32> {}
+unsafe impl Send for InsertMatrixIntoMatrix<i64, i64> {}
+unsafe impl Send for InsertMatrixIntoMatrix<f32, f32> {}
+unsafe impl Send for InsertMatrixIntoMatrix<f64, f64> {}
+
+unsafe impl Sync for InsertMatrixIntoMatrix<bool, bool> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<u8, u8> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<u16, u16> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<u32, u32> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<u64, u64> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<i8, i8> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<i16, i16> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<i32, i32> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<i64, i64> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<f32, f32> {}
+unsafe impl Sync for InsertMatrixIntoMatrix<f64, f64> {}
+
 #[derive(Debug, Clone)]
 pub struct InsertMatrixIntoMatrix<MatrixToInsertInto: ValueType, MatrixToInsert: ValueType> {
     _matrix_to_insert_into: PhantomData<MatrixToInsertInto>,
@@ -310,10 +337,10 @@ mod tests {
 
     use crate::context::{Context, Mode};
     use crate::operators::binary_operator::First;
-    use crate::sparse_matrix::{
+    use crate::util::ElementIndex;
+    use crate::value_types::sparse_matrix::{
         FromMatrixElementList, GetMatrixElementValue, MatrixElementList, Size,
     };
-    use crate::util::ElementIndex;
 
     #[test]
     fn test_insert_matrix_into_matrix() {
